from datetime import datetime

from django.db import IntegrityError
from django.utils.timezone import now, utc

from .models import JWTBlacklistToken


def jwt_blacklist_get_handler(payload):
    """
    Default implementation to check if a blacklisted jwt token exists.
    Should return a black listed token or None.
    """
    jti = payload.get('jti')

    try:
        token = JWTBlacklistToken.objects.get(jti=jti)
    except JWTBlacklistToken.DoesNotExist:
        return None
    else:
        return token


def jwt_blacklist_set_handler(payload):
    """
    Default implementation that blacklists a jwt token.
    Should return a black listed token or None.
    """
    try:
        data = {
            'jti': payload.get('jti'),
            'created': now(),
            'expires': datetime.fromtimestamp(payload.get('exp'), tz=utc)
        }
        token = JWTBlacklistToken.objects.create(**data)
    except (TypeError, IntegrityError, Exception):
        return None
    else:
        return token



from django import forms
from django.utils.translation import ugettext as _

from graphene_jwt_auth.forms import VerificationBaseForm
from graphene_jwt_auth.settings import api_settings
from .models import JWTLongRunningToken

jwt_long_running_token_get_handler = api_settings.JWT_LONG_RUNNING_TOKEN_GET_HANDLER
jwt_long_running_token_set_handler = api_settings.JWT_LONG_RUNNING_TOKEN_SET_HANDLER
jwt_using_long_running_token = api_settings.JWT_USING_LONG_RUNNING_TOKEN
jwt_payload_handler = api_settings.JWT_PAYLOAD_HANDLER
jwt_encode_handler = api_settings.JWT_ENCODE_HANDLER


class LongRunningJWTTokenForm(VerificationBaseForm):
    """
    Get long running token.
    """

    def clean(self):

        if not jwt_using_long_running_token:
            msg = _("Enable JWT_USING_LONG_RUNNING_TOKEN in your settings.")
            raise forms.ValidationError(msg)

        token = self.cleaned_data.get('token')

        # check payload and user
        payload = self._check_payload(token)
        user = self._check_user(payload)

        data = {'user': user}

        # check if long running token already exists
        long_running_token = jwt_long_running_token_get_handler(user)

        if long_running_token is None:
            long_running_token = jwt_long_running_token_set_handler(payload)

        data['token'] = long_running_token.key

        return data


class ObtainNewJWTTokenForm(forms.Form):
    """
    Get JWT Token based on long running token.
    """
    long_running_token = forms.CharField()

    def clean(self):
        if not jwt_using_long_running_token:
            msg = _("Enable JWT_USING_LONG_RUNNING_TOKEN in your settings.")
            raise forms.ValidationError(msg)

        long_running_token = self.cleaned_data.get('long_running_token')

        # check long running token on models
        try:
            token = JWTLongRunningToken.objects.get(key=long_running_token)
        except JWTLongRunningToken.DoesNotExist:
            msg = _("Long running token doesn't exists.")
            raise forms.ValidationError(msg)
        else:
            payload = jwt_payload_handler(token.user)

        return {
            'token': jwt_encode_handler(payload),
            'user': token.user
        }




from graphene_jwt_auth.compat import is_authenticated


class BasePermission(object):
    """
    A base class from which all permission classes should inherit.
    """

    def has_permission(self, request, view):
        """
        Return `True` if permission is granted, `False` otherwise.
        """
        return True

    def has_object_permission(self, request, view, obj):
        """
        Return `True` if permission is granted, `False` otherwise.
        """
        return True


class IsAuthenticated(BasePermission):
    def has_permission(self, request, view):
        return request.user and is_authenticated(request.user)


class IsAdminUser(BasePermission):
    def has_permission(self, request, view):
        return request.user and is_authenticated(request.user) and request.user.is_staff


class IsSuperUser(BasePermission):
    def has_permission(self, request, view):
        return request.user and is_authenticated(request.user) and request.user.is_superuser



from django.db import models
from django.utils.timezone import now

from graphene_jwt_auth.settings import api_settings


class JWTBlacklistToken(models.Model):
    jti = models.CharField(max_length=40, primary_key=True)
    created = models.DateTimeField(auto_now_add=True)
    expires = models.DateTimeField()

    class Meta:
        abstract = not api_settings.JWT_USING_BLACKLIST
        verbose_name = 'JWT Blacklist Token'
        verbose_name_plural = 'JWT Blacklist Tokens'

    def __str__(self):
        return self.jti

    def is_active(self):
        return self.expires > now()

    is_active.boolean = True
    is_active.short_description = 'active'



"""
Code taken from django-rest-framework
https://github.com/tomchristie/django-rest-framework/blob/master/rest_framework/settings.py
"""
import datetime
from importlib import import_module

from django.conf import settings
from django.test.signals import setting_changed
from django.utils import six

DEFAULTS = {
    'JWT_ENCODE_HANDLER': 'graphene_jwt_auth.payload.jwt_encode_handler',
    'JWT_DECODE_HANDLER': 'graphene_jwt_auth.payload.jwt_decode_handler',
    'JWT_PAYLOAD_HANDLER': 'graphene_jwt_auth.payload.jwt_payload_handler',
    'JWT_PAYLOAD_GET_USERNAME_HANDLER':
        'graphene_jwt_auth.payload.jwt_get_username_from_payload_handler',
    'JWT_SECRET_KEY': settings.SECRET_KEY,
    'JWT_PRIVATE_KEY': None,
    'JWT_PUBLIC_KEY': None,
    'JWT_ALGORITHM': 'HS256',
    'JWT_VERIFY': True,
    'JWT_VERIFY_EXPIRATION': True,
    'JWT_LEEWAY': 0,
    'JWT_EXPIRATION_DELTA': datetime.timedelta(seconds=300),
    'JWT_AUDIENCE': None,
    'JWT_ISSUER': None,
    'JWT_ALLOW_REFRESH': False,
    'JWT_REFRESH_EXPIRATION_DELTA': datetime.timedelta(days=7),
    'JWT_AUTH_HEADER_PREFIX': 'Bearer',

    # Blacklist
    # Don't override function below `JWT_USING_BLACKLIST`
    'JWT_USING_BLACKLIST': 'graphene_jwt_auth.blacklist' in settings.INSTALLED_APPS,
    'JWT_BLACKLIST_GET_HANDLER': 'graphene_jwt_auth.blacklist.utils.jwt_blacklist_get_handler',
    'JWT_BLACKLIST_SET_HANDLER': 'graphene_jwt_auth.blacklist.utils.jwt_blacklist_set_handler',


    # Long running token
    # Don't override function below `JWT_USING_LONG_RUNNING_TOKEN`
    'JWT_USING_LONG_RUNNING_TOKEN':
        'graphene_jwt_auth.longrunningtoken' in settings.INSTALLED_APPS,
    'JWT_LONG_RUNNING_TOKEN_GET_HANDLER':
        'graphene_jwt_auth.longrunningtoken.utils.jwt_long_running_token_get_handler',
    'JWT_LONG_RUNNING_TOKEN_SET_HANDLER':
        'graphene_jwt_auth.longrunningtoken.utils.jwt_long_running_token_set_handler',
    'JWT_LONG_RUNNING_TOKEN_APP_NAME': 'app',

    # Utils
    'JWT_DELETE_LONG_RUNNING_TOKEN_WHEN_LOGOUT': False,
    'CHANGED_PASSWORD_INVALIDATED_OLD_TOKEN': False,

    # Graphene
    'QUERIES_USER_NODE':
        'graphene_jwt_auth.graphene.queries.UserNode'
}

IMPORT_STRINGS = (
    'JWT_ENCODE_HANDLER',
    'JWT_DECODE_HANDLER',
    'JWT_PAYLOAD_HANDLER',
    'JWT_PAYLOAD_GET_USERNAME_HANDLER',
    'JWT_BLACKLIST_GET_HANDLER',
    'JWT_BLACKLIST_SET_HANDLER',
    'JWT_LONG_RUNNING_TOKEN_GET_HANDLER',
    'JWT_LONG_RUNNING_TOKEN_SET_HANDLER',
    'QUERIES_USER_NODE'
)


def perform_import(val, setting_name):
    """
    If the given setting is a string import notation,
    then perform the necessary import or imports.
    """
    if val is None:
        return None
    elif isinstance(val, six.string_types):
        return import_from_string(val, setting_name)
    elif isinstance(val, (list, tuple)):
        return [import_from_string(item, setting_name) for item in val]
    return val


def import_from_string(val, setting_name):
    """
    Attempt to import a class from a string representation.
    """
    try:
        # Nod to tastypie's use of importlib.
        parts = val.split('.')
        module_path, class_name = '.'.join(parts[:-1]), parts[-1]
        module = import_module(module_path)
        return getattr(module, class_name)
    except (ImportError, AttributeError) as e:
        msg = "Could not import '%s' for API setting '%s'. %s: %s." % (
            val, setting_name, e.__class__.__name__, e)
        raise ImportError(msg)


class APISettings(object):
    """
       A settings object, that allows API settings to be accessed as properties.
       For example:
           from rest_framework.settings import api_settings
           print(api_settings.DEFAULT_RENDERER_CLASSES)
       Any setting with string import paths will be automatically resolved
       and return the class, rather than the string literal.
       """

    def __init__(self, user_settings=None, defaults=None, import_strings=None):
        if user_settings:
            self._user_settings = self.__check_user_settings(user_settings)
        self.defaults = defaults or DEFAULTS
        self.import_strings = import_strings or IMPORT_STRINGS

    @property
    def user_settings(self):
        if not hasattr(self, '_user_settings'):
            self._user_settings = getattr(settings, 'GRAPHENE_JWT_AUTH', {})
        return self._user_settings

    def __getattr__(self, attr):
        if attr not in self.defaults:
            raise AttributeError("Invalid API setting: '%s'" % attr)

        try:
            # Check if present in user settings
            val = self.user_settings[attr]
        except KeyError:
            # Fall back to defaults
            val = self.defaults[attr]

        # Coerce import strings into classes
        if attr in self.import_strings:
            val = perform_import(val, attr)

        # Cache the result
        setattr(self, attr, val)
        return val

    def __check_user_settings(self, user_settings):
        # not needed yet
        # for item in IMPORTANT_INSTALLED_APPS.items():
        #     if self.defaults[item[0]] and not item[1] in settings.INSTALLED_APPS:
        #         raise RuntimeError(
        #             "Please put '%s' on your settings.INSTALLED_APPS because you enable '%s'." %
        #             (item[1], item[0]))
        return user_settings


api_settings = APISettings(None, DEFAULTS, IMPORT_STRINGS)


def reload_api_settings(*args, **kwargs):
    global api_settings
    setting, value = kwargs['setting'], kwargs['value']
    if setting == 'GRAPHENE_JWT_AUTH':
        api_settings = APISettings(value, DEFAULTS, IMPORT_STRINGS)


setting_changed.connect(reload_api_settings)



"""
graphene-jwt-auth library
-----------------------

Several utilities to implement JWT Authentication in Django and Graphene.

:copyright: (c) 2017 by Darwin Herlinmanda Cahyadi
:license: MIT. See LICENSE for more details
"""

__title__ = 'graphene_jwt_auth'
__version__ = '0.0.1'
__author__ = 'Darwin Herlinmanda Cahyadi'
__license__ = 'MIT'
__copyright__ = 'Copyright 2017 Darwin Herlinmanda Cahyadi'



from django.conf import settings
from django.db import models
from django.utils.timezone import now

from graphene_jwt_auth.settings import api_settings
from graphene_jwt_auth.utils import generate_key


class JWTLongRunningToken(models.Model):
    key = models.CharField('Key', max_length=40, primary_key=True)
    user = models.OneToOneField(
        settings.AUTH_USER_MODEL, related_name='long_running_tokens',
        on_delete=models.CASCADE, verbose_name='User'
    )
    app = models.CharField(max_length=255)
    created = models.DateTimeField("Created", auto_now_add=True)
    # expires taken from JWT_REFRESH_EXPIRATION_DELTA
    expires = models.DateTimeField()

    class Meta:
        abstract = not api_settings.JWT_USING_LONG_RUNNING_TOKEN
        verbose_name = "Long Running Token"
        verbose_name_plural = "Long Running Tokens"

    def save(self, *args, **kwargs):
        if not self.key:
            self.key = generate_key()
        return super().save(*args, **kwargs)

    def __str__(self):
        return self.key

    def is_active(self):
        return self.expires > now()

    is_active.boolean = True
    is_active.short_description = 'active'



from django.contrib import admin

from graphene_jwt_auth.longrunningtoken.models import JWTLongRunningToken
from graphene_jwt_auth.settings import api_settings


class JWTLongRunningTokenAdmin(admin.ModelAdmin):
    list_display = ('user', 'key', 'created', 'expires', 'is_active')
    fields = ('user', 'key', 'app', 'created', 'expires', 'is_active')
    readonly_fields = ('user', 'key', 'app', 'created', 'expires', 'is_active')


if api_settings.JWT_USING_LONG_RUNNING_TOKEN:
    admin.site.register(JWTLongRunningToken, JWTLongRunningTokenAdmin)



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-02-28 05:02
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='JWTBlacklistToken',
            fields=[
                ('jti', models.CharField(max_length=40, primary_key=True, serialize=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('expires', models.DateTimeField()),
            ],
            options={
                'verbose_name_plural': 'JWT Blacklist Tokens',
                'verbose_name': 'JWT Blacklist Token',
                'abstract': False,
            },
        ),
    ]



from django.core.serializers.json import DjangoJSONEncoder
from django.http import HttpResponse, HttpResponseBadRequest
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext as _
from django.views.decorators.csrf import csrf_exempt
from django.views.generic import View

from graphene_jwt_auth.compat import json, smart_text
from graphene_jwt_auth.forms import JSONWebTokenForm


class ObtainJSONWebToken(View):
    http_method_names = ['post']
    error_response_dict = {'errors': [_("Improperly formatted request")]}
    json_encoder_class = DjangoJSONEncoder

    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        return super(ObtainJSONWebToken, self).dispatch(request, *args, **kwargs)

    def post(self, request, *args, **kwargs):
        try:
            request_json = json.loads(smart_text(request.body))
        except ValueError:
            return self.render_bad_request_response()

        form = JSONWebTokenForm(request_json)

        if not form.is_valid():
            return self.render_bad_request_response({'errors': form.errors})

        context_dict = {
            'token': form.object['token']
        }

        return self.render_response(context_dict)

    def render_response(self, context_dict):
        json_context = json.dumps(context_dict, cls=self.json_encoder_class)

        return HttpResponse(json_context, content_type='application/json')

    def render_bad_request_response(self, error_dict=None):
        if error_dict is None:
            error_dict = self.error_response_dict

        json_context = json.dumps(error_dict, cls=self.json_encoder_class)

        return HttpResponseBadRequest(
            json_context, content_type='application/json')


obtain_jwt_token = ObtainJSONWebToken.as_view()



from django.contrib.auth.models import AnonymousUser
from django.utils.functional import SimpleLazyObject

from graphene_jwt_auth.authentication import JSONWebTokenAuthentication
from graphene_jwt_auth.compat import is_authenticated


def get_user_jwt(request):
    # check first if request.user is available
    # if django still use session
    user = JSONWebTokenAuthentication().authenticate(request)
    return user[0] or AnonymousUser()


class DjangoJWTAuthenticationMiddleware(object):
    """
    Token base authentication using the JSON Web Token standard.
    This middleware used in django middleware.

    Clients should authenticate by passing the token key in "Authorization"
    HTTP header, prepended with the string specified in the setting
    `JWT_AUTH_HEADER_PREFIX`. For example:
        Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.
        eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiYWRtaW4iOnRydWV9.
        TJVA95OrM7E2cBab30RMHrHDcEfxjoYZgeFONFh7HgQ
    """

    def __init__(self, get_response):
        self.get_response = get_response

    def __call__(self, request):
        if not (request.user and is_authenticated(request.user)):
            request.user = SimpleLazyObject(lambda: get_user_jwt(request))

        return self.get_response(request)


class GrapheneJWTAuthenticationMiddleware(object):
    """
    Token base authentication using the JSON Web Token standard.
    This middleware used in graphene middleware.

     Clients should authenticate by passing the token key in "Authorization"
    HTTP header, prepended with the string specified in the setting
    `JWT_AUTH_HEADER_PREFIX`. For example:
        Authorization: Bearer eyJhbGciOiJIUzI1NiIsInR5cCI6IkpXVCJ9.
        eyJzdWIiOiIxMjM0NTY3ODkwIiwibmFtZSI6IkpvaG4gRG9lIiwiYWRtaW4iOnRydWV9.
        TJVA95OrM7E2cBab30RMHrHDcEfxjoYZgeFONFh7HgQ
    """
    def resolve(self, next, root, args, context, info):
        if not (context.user and is_authenticated(context.user)):
            context.user = SimpleLazyObject(lambda: get_user_jwt(context))

        return next(root, args, context, info)



from django.contrib import admin

from graphene_jwt_auth.blacklist.models import JWTBlacklistToken
from graphene_jwt_auth.settings import api_settings


class JWTBlacklistTokenAdmin(admin.ModelAdmin):
    list_display = ('jti', 'expires', 'created', 'is_active')
    fields = ('jti', 'expires', 'created', 'is_active')
    readonly_fields = ('jti', 'expires', 'created', 'is_active')


if api_settings.JWT_USING_BLACKLIST:
    admin.site.register(JWTBlacklistToken, JWTBlacklistTokenAdmin)



import six
from graphene.relay.mutation import ClientIDMutationMeta
from graphene.types.objecttype import ObjectType
from promise import Promise

from graphene_jwt_auth.compat import is_authenticated
from graphene_jwt_auth.exceptions import NotAuthenticated, PermissionDenied


class ClientIDMutation(six.with_metaclass(ClientIDMutationMeta, ObjectType)):
    permission_classes = []

    @classmethod
    def mutate(cls, root, args, context, info):

        # Check permissions first
        # TODO : still need improve this mutation with auth
        # im not happy with this implementation, i don't know why i can't use super on this method
        cls.check_permission(context)

        input = args.get('input')

        def on_resolve(payload):
            try:
                payload.client_mutation_id = input.get('clientMutationId')
            except:
                raise Exception((
                    'Cannot set client_mutation_id in the payload object {}'
                ).format(repr(payload)))
            return payload

        return Promise.resolve(
            cls.mutate_and_get_payload(input, context, info)
        ).then(on_resolve)

    @classmethod
    def get_permissions(cls):
        """
        Instantiates and returns the list of permissions that this view requires.
        """
        return [permission() for permission in cls.permission_classes]

    @classmethod
    def check_permission(cls, context):
        """
        Check if the request should be premitted.
        Raises an appropriate exception if the request is not permitted.
        """
        for permission in cls.get_permissions():
            if not permission.has_permission(context, cls):
                cls.permission_denied(context)

    @classmethod
    def permission_denied(cls, context):
        if not (context.user and is_authenticated(context.user)):
            raise NotAuthenticated()
        raise PermissionDenied()



from __future__ import unicode_literals

import binascii
import os

from django.utils.encoding import smart_text
from django.utils.translation import ugettext as _

from graphene_jwt_auth import exceptions
from graphene_jwt_auth.settings import api_settings


def get_authorization_header(request):
    """
    Return request's 'Authorization:' header, as a bytestring.
    From: https://github.com/tomchristie/django-rest-framework/blob/master/rest_framework/authentication.py
    """
    auth = request.META.get('HTTP_AUTHORIZATION', b'')

    if isinstance(auth, type('')):
        # Work around django test client oddness
        auth = auth.encode('iso-8859-1')

    return auth


def generate_key():
    return binascii.hexlify(os.urandom(20)).decode()


def get_token(request):
    auth = get_authorization_header(request).split()
    auth_header_prefix = api_settings.JWT_AUTH_HEADER_PREFIX.lower()

    if not auth or smart_text(auth[0].lower()) != auth_header_prefix:
        return None

    if len(auth) == 1:
        msg = _('Invalid Authorization header. No credentials provided.')
        raise exceptions.AuthenticationFailed(msg)
    elif len(auth) > 2:
        msg = _('Invalid Authorization header. Credentials string '
                'should not contain spaces.')
        raise exceptions.AuthenticationFailed(msg)

    return auth[1]



import graphene
from django.contrib.auth.signals import user_logged_in
from django.core.exceptions import ValidationError
from django.utils.translation import ugettext as _
from graphene import relay

from graphene_jwt_auth.blacklist.forms import BlacklistJWTTokenForm
from graphene_jwt_auth.exceptions import AuthenticationFailed
from graphene_jwt_auth.forms import (
    JSONWebTokenForm,
    RefreshJSONWebTokenForm,
    VerifyJSONWebTokenForm
)
from graphene_jwt_auth.longrunningtoken.forms import LongRunningJWTTokenForm, ObtainNewJWTTokenForm
from graphene_jwt_auth.settings import api_settings
from graphene_jwt_auth.utils.core import get_token
from .permissions import IsAuthenticated
from .relay.mutation import ClientIDMutation

user_node = api_settings.QUERIES_USER_NODE
jwt_decode_handler = api_settings.JWT_DECODE_HANDLER
jwt_using_blacklist = api_settings.JWT_USING_BLACKLIST
jwt_blacklist_set_handler = api_settings.JWT_BLACKLIST_SET_HANDLER
jwt_blacklist_get_handler = api_settings.JWT_BLACKLIST_GET_HANDLER
jwt_using_long_running_token = api_settings.JWT_USING_LONG_RUNNING_TOKEN
jwt_long_running_token_get_handler = api_settings.JWT_LONG_RUNNING_TOKEN_GET_HANDLER
jwt_delete_long_running_token_when_logout = api_settings.JWT_DELETE_LONG_RUNNING_TOKEN_WHEN_LOGOUT


class Login(relay.ClientIDMutation):

    class Input:
        email = graphene.String(required=True)
        password = graphene.String(required=True)

    token = graphene.String()
    user = graphene.Field(user_node)

    @classmethod
    def mutate_and_get_payload(cls, input, context, info):
        data = {
            'email': input.get('email'),
            'password': input.get('password')
        }

        form = JSONWebTokenForm(data)

        if not form.is_valid():
            raise ValidationError(form.errors)

        user = form.cleaned_data.get('user')
        user_logged_in.send(sender=user.__class__, request=context, user=user)
        return Login(**form.cleaned_data)


class RefreshToken(ClientIDMutation):
    permission_classes = (IsAuthenticated,)

    token = graphene.String()
    user = graphene.Field(user_node)

    @classmethod
    def mutate_and_get_payload(cls, input, context, info):
        token = get_token(context)

        if not token:
            msg = _("Authorization header not found.")
            raise AuthenticationFailed(msg)

        form = RefreshJSONWebTokenForm({"token": token})

        if not form.is_valid():
            raise ValidationError(form.errors)

        return RefreshToken(**form.cleaned_data)


class VerifyToken(ClientIDMutation):

    class Input:
        token = graphene.String(required=True)

    token = graphene.String()
    user = graphene.Field(user_node)

    @classmethod
    def mutate_and_get_payload(cls, input, context, info):
        token = input.get('token')

        form = VerifyJSONWebTokenForm({'token': token})

        if not form.is_valid():
            raise ValidationError(form.errors)

        return VerifyToken(**form.cleaned_data)


class LongRunningToken(ClientIDMutation):
    permission_classes = (IsAuthenticated,)

    long_running_token = graphene.String()
    user = graphene.Field(user_node)

    @classmethod
    def mutate_and_get_payload(cls, input, context, info):
        token = get_token(context)

        if not token:
            msg = _("Authorization header not found.")
            raise AuthenticationFailed(msg)

        form = LongRunningJWTTokenForm({"token": token})

        if not form.is_valid():
            raise ValidationError(form.errors)

        data = {
            'long_running_token': form.cleaned_data.get('token'),
            'user': form.cleaned_data.get('user')
        }

        return LongRunningToken(**data)


class ObtainNewToken(ClientIDMutation):

    class Input:
        long_running_token = graphene.String(required=True)

    token = graphene.String()
    user = graphene.Field(user_node)

    @classmethod
    def mutate_and_get_payload(cls, input, context, info):
        input_data = {
            'long_running_token': input.get('long_running_token')
        }

        form = ObtainNewJWTTokenForm(input_data)

        if not form.is_valid():
            raise ValidationError(form.errors)

        data = {
            'token': form.cleaned_data.get('token'),
            'user': form.cleaned_data.get('user')
        }

        return ObtainNewToken(**data)


class Logout(ClientIDMutation):
    status = graphene.String()

    @classmethod
    def mutate_and_get_payload(cls, input, context, info):
        token = get_token(context)

        if not token:
            return Logout(status='ok')

        form = BlacklistJWTTokenForm({'token': token})

        if not form.is_valid():
            return Logout(status='Ok')

        payload = form.cleaned_data.get('payload')
        user = form.cleaned_data.get('user')

        if jwt_using_blacklist:
            blacklist = jwt_blacklist_get_handler(payload)

            if blacklist is None:
                jwt_blacklist_set_handler(payload)

        if jwt_using_long_running_token and jwt_delete_long_running_token_when_logout:
            long_running_token = jwt_long_running_token_get_handler(user)

            if long_running_token is not None:
                long_running_token.delete()

        return Logout(status='Ok')


class DeleteLongRunningToken(ClientIDMutation):
    status = graphene.String()

    @classmethod
    def mutate_and_get_payload(cls, input, context, info):
        token = get_token(context)

        if not token:
            return DeleteLongRunningToken(status='ok')

        form = BlacklistJWTTokenForm({'token': token})

        if not form.is_valid():
            return DeleteLongRunningToken(status='Ok')

        user = form.cleaned_data.get('user')
        long_running_token = jwt_long_running_token_get_handler(user)

        if long_running_token is not None:
            long_running_token.delete()

        return DeleteLongRunningToken(status='ok')


class Mutation(graphene.AbstractType):
    login = Login.Field()
    refresh_token = RefreshToken.Field()
    verify_token = VerifyToken.Field()
    long_running_token = LongRunningToken.Field()
    delete_long_running_token = DeleteLongRunningToken.Field()
    obtain_new_token = ObtainNewToken.Field()
    logout = Logout.Field()



from django import forms
from django.utils.translation import ugettext as _

from graphene_jwt_auth.forms import VerificationBaseForm
from graphene_jwt_auth.settings import api_settings

jwt_using_blacklist = api_settings.JWT_USING_BLACKLIST
jwt_blacklist_get_handler = api_settings.JWT_BLACKLIST_GET_HANDLER
jwt_blacklist_set_handler = api_settings.JWT_BLACKLIST_SET_HANDLER


class BlacklistJWTTokenForm(VerificationBaseForm):
    """
    Blacklist JWT Token and long running token.
    """

    def clean(self):

        if not jwt_using_blacklist:
            msg = _("Enable JWT_USING_BLACKLIST in your settings.")
            raise forms.ValidationError(msg)

        token = self.cleaned_data.get('token')

        # check payload and user
        payload = self._check_payload(token)
        user = self._check_user(payload)

        return {
            'payload': payload,
            'user': user
        }



# Create your views here.
import six
from graphene_django.views import GraphQLView
from graphql.error import GraphQLError

from .utils import format_error as format_graphql_error


class CustomGraphQLView(GraphQLView):

    @staticmethod
    def format_error(error):
        if isinstance(error, GraphQLError):
            return format_graphql_error(error)

        return {'message': six.text_type(error)}



from graphene import AbstractType, relay
from graphene_django.filter import DjangoFilterConnectionField
from graphene_django.types import DjangoObjectType

from graphene_jwt_auth.compat import User


class UserNode(DjangoObjectType):
    class Meta:
        model = User
        # we not use only_fields because too much
        exclude_fields = ('password',)
        interfaces = (relay.Node,)


class Query(AbstractType):
    user = relay.Node.Field(UserNode)
    users = DjangoFilterConnectionField(UserNode)



import jwt
from django.http import HttpResponse
from django.utils.decorators import method_decorator
from django.utils.translation import ugettext as _
from django.views.decorators.csrf import csrf_exempt

from graphene_jwt_auth import exceptions
from graphene_jwt_auth.compat import User, json, smart_text
from graphene_jwt_auth.settings import api_settings
from graphene_jwt_auth.utils import get_authorization_header

jwt_decode_handler = api_settings.JWT_DECODE_HANDLER
jwt_get_user_id_from_payload = api_settings.JWT_PAYLOAD_GET_USER_ID_HANDLER


class JSONWebTokenAuthMixin(object):
    """
    Token based authentication using the JSON Web Token standard.

    Clients should authenticate by passing the token key in the "Authorization"
    HTTP header, prepended with the string specified in the setting
    `JWT_AUTH_HEADER_PREFIX`. For example:

        Authorization: JWT eyJhbGciOiAiSFMyNTYiLCAidHlwIj
    """
    www_authenticate_realm = 'api'

    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        try:
            request.user, request.token = self.authenticate(request)
        except exceptions.AuthenticationFailed as e:
            response = HttpResponse(
                json.dumps({'errors': [str(e)]}),
                status=401,
                content_type='application/json'
            )

            response['WWW-Authenticate'] = self.authenticate_header(request)

            return response

        return super(JSONWebTokenAuthMixin, self).dispatch(
            request, *args, **kwargs)

    def authenticate(self, request):
        auth = get_authorization_header(request).split()
        auth_header_prefix = api_settings.JWT_AUTH_HEADER_PREFIX.lower()

        if not auth or smart_text(auth[0].lower()) != auth_header_prefix:
            raise exceptions.AuthenticationFailed()

        if len(auth) == 1:
            msg = _("Invalid Authorization header. No credentials provided.")
            raise exceptions.AuthenticationFailed(msg)
        elif len(auth) > 2:
            msg = _("Invalid Authorization header. Credentials string should not contain spaces.")
            raise exceptions.AuthenticationFailed(msg)

        try:
            payload = jwt_decode_handler(auth[1])
        except jwt.ExpiredSignature:
            msg = _("Signature has expired.")
            raise exceptions.AuthenticationFailed(msg)
        except jwt.DecodeError:
            msg = _("Error decoding signature.")
            raise exceptions.AuthenticationFailed(msg)

        user = self.authenticate_credentials(payload)

        return (user, auth[1])

    def authenticate_credentials(self, payload):
        """
        Returns an active user that matches the payload's user id and email.
        """
        try:
            user_id = jwt_get_user_id_from_payload(payload)

            if user_id:
                user = User.objects.get(pk=user_id, is_active=True)
            else:
                msg = _("Invalid payload.")
                raise exceptions.AuthenticationFailed(msg)
        except User.DoesNotExist:
            msg = _("Invalid signature")
            raise exceptions.AuthenticationFailed(msg)

        return user

    def authenticate_header(self, request):
        """
        Return a string to be used as the value of the `WWW-Authenticate`
        header in a `401 Unauthenticated` response, or `None` if the
        authentication scheme should return `403 Permission Denied` responses.
        """
        return 'JWT realm="{0}"'.format(self.www_authenticate_realm)



from calendar import timegm
from datetime import datetime, timedelta

import jwt
from django import forms
from django.contrib.auth import authenticate
from django.utils.translation import ugettext as _

from graphene_jwt_auth.compat import User, get_username_field
from graphene_jwt_auth.settings import api_settings

jwt_payload_handler = api_settings.JWT_PAYLOAD_HANDLER
jwt_encode_handler = api_settings.JWT_ENCODE_HANDLER
jwt_decode_handler = api_settings.JWT_DECODE_HANDLER
jwt_get_username_from_payload = api_settings.JWT_PAYLOAD_GET_USERNAME_HANDLER


class JSONWebTokenForm(forms.Form):
    password = forms.CharField()

    def __init__(self, *args, **kwargs):
        super(JSONWebTokenForm, self).__init__(*args, **kwargs)

        # Dynamically add the USERNAME_FIELD to self.fields.
        self.fields[self.username_field] = forms.CharField()
        self.object = {}

    @property
    def username_field(self):
        return get_username_field()

    def clean(self):
        cleaned_data = super(JSONWebTokenForm, self).clean()
        credentials = {
            self.username_field: cleaned_data.get(self.username_field),
            'password': cleaned_data.get('password')
        }

        if all(credentials.values()):
            user = authenticate(**credentials)

            if user:
                if not user.is_active:
                    msg = _("User account is disabled.")
                    raise forms.ValidationError(msg)

                payload = jwt_payload_handler(user)

                # add token and user to cleaned_data
                return {
                    'token': jwt_encode_handler(payload),
                    'user': user
                }
            else:
                msg = _("Unable to login with provided credentials.")
                raise forms.ValidationError(msg)
        else:
            msg = _("Must include '{username_field}' and 'password'")
            msg = msg.format(username_field=self.username_field)
            raise forms.ValidationError(msg)


class VerificationBaseForm(forms.Form):
    token = forms.CharField()

    def clean(self):
        msg = _("Please define a clean method.")
        raise NotImplementedError(msg)

    @staticmethod
    def _check_payload(token):
        # Check payload valid
        try:
            payload = jwt_decode_handler(token)
        except jwt.ExpiredSignature:
            msg = _("Signature has expired.")
            raise forms.ValidationError(msg)
        except jwt.DecodeError:
            msg = _("Error decoding signature.")
            raise forms.ValidationError(msg)

        return payload

    @staticmethod
    def _check_user(payload):
        username = jwt_get_username_from_payload(payload)

        if not username:
            msg = _("Invalid payload.")
            raise forms.ValidationError(msg)

        try:
            user = User.objects.get_by_natural_key(username)
        except User.DoesNotExist:
            msg = _("User doesn't exists.")
            raise forms.ValidationError(msg)

        if not user.is_active:
            msg = _("User account is disabled.")
            raise forms.ValidationError(msg)

        return user


class VerifyJSONWebTokenForm(VerificationBaseForm):
    """
    Check the veracity of an access token.
    """

    def clean(self):
        token = self.cleaned_data.get('token')

        payload = self._check_payload(token)
        user = self._check_user(payload)

        return {
            'token': token,
            'user': user
        }


class RefreshJSONWebTokenForm(VerificationBaseForm):
    """
    Refresh an access token.
    """

    def clean(self):
        token = self.cleaned_data.get('token')

        payload = self._check_payload(token)
        user = self._check_user(payload)

        # Get and check 'orig_iat'
        orig_iat = payload.get('orig_iat')
        jti = payload.get('jti')

        if orig_iat:
            # Verify expiration
            refresh_limit = api_settings.JWT_REFRESH_EXPIRATION_DELTA

            if isinstance(refresh_limit, timedelta):
                refresh_limit = (refresh_limit.days * 24 * 3600 + refresh_limit.seconds)

            expiration_timestamp = orig_iat + int(refresh_limit)
            now_timestamp = timegm(datetime.utcnow().utctimetuple())

            if now_timestamp > expiration_timestamp:
                msg = _("Refresh has expired.")
                raise forms.ValidationError(msg)

        else:
            msg = _("orig_iat field is required.")
            raise forms.ValidationError(msg)

        new_payload = jwt_payload_handler(user)
        new_payload['orig_iat'] = orig_iat

        if jti:
            new_payload['jti'] = jti

        return {
            'token': jwt_encode_handler(new_payload),
            'user': user
        }



import jwt
from django.utils.encoding import smart_text
from django.utils.translation import ugettext as _

from graphene_jwt_auth.compat import User
from graphene_jwt_auth.exceptions import AuthenticationFailed
from graphene_jwt_auth.settings import api_settings
from graphene_jwt_auth.utils import get_authorization_header
from django.utils.dateformat import format

changed_password_invalidated_old_token = api_settings.CHANGED_PASSWORD_INVALIDATED_OLD_TOKEN
jwt_decode_handler = api_settings.JWT_DECODE_HANDLER
jwt_get_username_from_payload = api_settings.JWT_PAYLOAD_GET_USERNAME_HANDLER
jwt_blacklist_get_handler = api_settings.JWT_BLACKLIST_GET_HANDLER
jwt_using_blacklist = api_settings.JWT_USING_BLACKLIST


class BaseJSONWebTokenAuthentication(object):
    """
    Token based authentication using the JSON Web Token standard.
    """

    def authenticate(self, request):
        jwt_value = self.get_jwt_value(request)

        if jwt_value is None:
            return None, None

        try:
            payload = jwt_decode_handler(jwt_value)
        except jwt.ExpiredSignature:
            msg = _("Signature has expired.")
            raise AuthenticationFailed(msg)
        except jwt.DecodeError:
            msg = _("Error decoding signature.")
            raise AuthenticationFailed(msg)
        except jwt.InvalidTokenError:
            raise AuthenticationFailed()

        if jwt_using_blacklist:
            # will hit database once
            blacklisted = jwt_blacklist_get_handler(payload)

            if blacklisted:
                msg = _("Token has been blacklisted.")
                raise AuthenticationFailed(msg)

        user = self.authenticate_credentials(payload)

        if changed_password_invalidated_old_token and hasattr(user, 'password_last_changed'):
            # will hit database once
            # need implement `password_last_changed` on user models
            orig_iat = int(payload.get('orig_iat'))
            password_last_changed = int(format(user.password_last_changed, 'U'))

            if orig_iat < password_last_changed:
                msg = _("User must re-authenticate after changing password.")
                raise AuthenticationFailed(msg)

        return user, jwt_value

    def authenticate_credentials(self, payload):
        username = jwt_get_username_from_payload(payload)

        if not username:
            msg = _("Invalid payload.")
            raise AuthenticationFailed(msg)

        try:
            user = User.objects.get_by_natural_key(username)
        except User.DoesNotExist:
            msg = _("Invalid signature.")
            raise AuthenticationFailed(msg)

        if not user.is_active:
            msg = _("User account is disabled.")
            raise AuthenticationFailed(msg)

        return user


class JSONWebTokenAuthentication(BaseJSONWebTokenAuthentication):
    """
        Clients should authenticate by passing the token key in the "Authorization"
        HTTP header, prepended with the string specified in the setting
        `JWT_AUTH_HEADER_PREFIX`. For example:

            Authorization: JWT eyJhbGciOiAiSFMyNTYiLCAidHlwIj
        """
    www_authenticate_realm = 'api'

    def get_jwt_value(self, request):
        auth = get_authorization_header(request).split()
        auth_header_prefix = api_settings.JWT_AUTH_HEADER_PREFIX.lower()

        if not auth or smart_text(auth[0].lower()) != auth_header_prefix:
            return None

        if len(auth) == 1:
            msg = _("Invalid Authorization header. No credentials provided.")
            raise AuthenticationFailed(msg)
        elif len(auth) > 2:
            msg = _("Invalid Authorization header. Credentials string should not contain spaces.")
            raise AuthenticationFailed(msg)

        return auth[1]

    def authenticate_header(self, request):
        """
        Return a string to be used as the value of the `WWW-Authenticate`
        header in a `401 Unauthenticated` response, or `None` if the
        authentication scheme should return `403 Permission Denied` responses.
        """
        return '{0} realm="{1}"'.format(api_settings.JWT_AUTH_HEADER_PREFIX,
                                        self.www_authenticate_realm)



from calendar import timegm
from datetime import datetime

import jwt

from graphene_jwt_auth.compat import get_username, get_username_field
from graphene_jwt_auth.settings import api_settings
from graphene_jwt_auth.utils import generate_key


def jwt_payload_handler(user):
    username_field = get_username_field()
    username = get_username(user)

    payload = {
        username_field: username,
        'exp': datetime.utcnow() + api_settings.JWT_EXPIRATION_DELTA
    }

    # Include original issued at time for a brand new token,
    # to allow token refresh
    if api_settings.JWT_ALLOW_REFRESH:
        payload['orig_iat'] = timegm(datetime.utcnow().utctimetuple())

    if api_settings.JWT_AUDIENCE is not None:
        payload['aud'] = api_settings.JWT_AUDIENCE

    if api_settings.JWT_ISSUER is not None:
        payload['iss'] = api_settings.JWT_ISSUER

    if api_settings.JWT_USING_BLACKLIST:
        payload['jti'] = generate_key()

    return payload


def jwt_get_username_from_payload_handler(payload):
    """
    Override this function if username is formatted differently in payload
    """
    return payload.get(get_username_field())


def jwt_encode_handler(payload):
    return jwt.encode(
        payload,
        api_settings.JWT_PRIVATE_KEY or api_settings.JWT_SECRET_KEY,
        api_settings.JWT_ALGORITHM
    ).decode('utf-8')


def jwt_decode_handler(token):
    options = {
        'verify_exp': api_settings.JWT_VERIFY_EXPIRATION,
    }

    return jwt.decode(
        token,
        api_settings.JWT_PUBLIC_KEY or api_settings.JWT_SECRET_KEY,
        api_settings.JWT_VERIFY,
        options=options,
        leeway=api_settings.JWT_LEEWAY,
        audience=api_settings.JWT_AUDIENCE,
        issuer=api_settings.JWT_ISSUER,
        algorithms=[api_settings.JWT_ALGORITHM]
    )



from datetime import datetime, timedelta

from django.core.exceptions import ValidationError
from django.db import IntegrityError
from django.utils.timezone import utc
from django.utils.translation import ugettext as _

from graphene_jwt_auth.compat import User
from graphene_jwt_auth.settings import api_settings
from .models import JWTLongRunningToken

jwt_get_username_from_payload = api_settings.JWT_PAYLOAD_GET_USERNAME_HANDLER


def jwt_long_running_token_get_handler(user):
    """
    Default implementation to check if a long running token exists.
    Should return long running token or None.
    """

    try:
        token = JWTLongRunningToken.objects.get(user=user)
    except JWTLongRunningToken.DoesNotExist:
        return None
    else:
        return token


def jwt_long_running_token_set_handler(payload):
    """
    Default implementation that create long running token.
    Should return a long running token or None.
    """

    # get username
    username = jwt_get_username_from_payload(payload)

    user = User.objects.get_by_natural_key(username)

    # Get and check 'orig_iat'
    orig_iat = payload.get('orig_iat')

    if orig_iat:
        # check refresh_limit
        refresh_limit = api_settings.JWT_REFRESH_EXPIRATION_DELTA
        if isinstance(refresh_limit, timedelta):
            refresh_limit = (refresh_limit.days * 24 * 3600 + refresh_limit.seconds)
        expiration_timestamp = orig_iat + int(refresh_limit)
    else:
        msg = _("orig_iat field is required.")
        raise ValidationError(msg)

    try:
        data = {
            'user': user,
            'app': api_settings.JWT_LONG_RUNNING_TOKEN_APP_NAME,
            'expires': datetime.fromtimestamp(expiration_timestamp, tz=utc)
        }
        token = JWTLongRunningToken.objects.create(**data)
    except(TypeError, IntegrityError, Exception):
        return None
    else:
        return token



from django.core import exceptions

from graphene_jwt_auth.exceptions import APIException


def format_error(error):
    """
    change messages error
    """
    data = {
        'code': 'unhandled_exception',
        'message': error.message,
        'status_code': 500,
    }

    if hasattr(error, 'original_error'):
        original_error = error.original_error

        if isinstance(original_error, APIException):
            data['code'] = original_error.get_codes()
            data['message'] = original_error.detail
            data['status_code'] = original_error.status_code

        if isinstance(original_error, exceptions.ValidationError):
            data['code'] = 'invalid'
            data['status_code'] = 400
            if hasattr(original_error, 'error_dict'):
                data['message'] = original_error.message_dict

    if error.locations is not None:
        data['locations'] = [
            {'line': loc.line, 'column': loc.column}
            for loc in error.locations
            ]

    return data
